@@ -3,6 +3,7 @@
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
  * Copyright 2001 Sun Microsystems Inc.
+ *           2002 Ximian Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,10 +25,10 @@
 #include <libbonobo.h>
 #include "../cspi-lowlevel.h"
 
-void
+CORBA_Object
 cspi_dup_ref (CORBA_Object object)
 {
-  bonobo_object_dup_ref (object, NULL);
+  return bonobo_object_dup_ref (object, NULL);
 }
 
 void
@@ -109,6 +110,12 @@ cspi_init (void)
   return registry;
 }
 
+SPIBoolean
+cspi_ping (CORBA_Object object)
+{
+	return bonobo_unknown_ping (object, NULL);
+}
+
 void
 cspi_main (void)
 {
