@@ -1,3 +1,25 @@
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
+ * Copyright 2001 Sun Microsystems Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <stdlib.h> /* for malloc */
 #include <cspi/spi-private.h>
 
@@ -103,7 +125,7 @@ cspi_init_role_table (AccessibleRole *role_table)
   role_table [Accessibility_ROLE_DIAL] = SPI_ROLE_DIAL;
   role_table [Accessibility_ROLE_DIALOG] = SPI_ROLE_DIALOG;
   role_table [Accessibility_ROLE_DIRECTORY_PANE] = SPI_ROLE_DIRECTORY_PANE;
-  role_table[Accessibility_ROLE_DRAWING_AREA] = SPI_ROLE_DRAWING_AREA;
+  role_table [Accessibility_ROLE_DRAWING_AREA] = SPI_ROLE_DRAWING_AREA;
   role_table [Accessibility_ROLE_FILE_CHOOSER] = SPI_ROLE_FILE_CHOOSER;
   role_table [Accessibility_ROLE_FILLER] = SPI_ROLE_FILLER;
   role_table [Accessibility_ROLE_FONT_CHOOSER] = SPI_ROLE_FONT_CHOOSER;
@@ -304,8 +326,7 @@ Accessible_getName (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, NULL);
 
-  retval = (char *)
-    Accessibility_Accessible__get_name (CSPI_OBJREF (obj), cspi_ev ());
+  retval = Accessibility_Accessible__get_name (CSPI_OBJREF (obj), cspi_ev ());
 
   cspi_return_val_if_ev ("getName", NULL); 
 
@@ -328,8 +349,7 @@ Accessible_getDescription (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, NULL);
 
-  retval = (char *)
-    Accessibility_Accessible__get_description (CSPI_OBJREF (obj),
+  retval = Accessibility_Accessible__get_description (CSPI_OBJREF (obj),
 					       cspi_ev ());
 
   cspi_return_val_if_ev ("getDescription", NULL); 
@@ -380,8 +400,7 @@ Accessible_getChildCount (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, -1);
 
-  retval = (long) 
-    Accessibility_Accessible__get_childCount (CSPI_OBJREF (obj),
+  retval = Accessibility_Accessible__get_childCount (CSPI_OBJREF (obj),
 					      cspi_ev ());
 
   cspi_return_val_if_ev ("getChildCount", -1); 
@@ -411,6 +430,7 @@ Accessible_getChildAtIndex (Accessible *obj,
     Accessibility_Accessible_getChildAtIndex (CSPI_OBJREF (obj),
 					      childIndex, cspi_ev ()));
 
+  cspi_return_val_if_ev ("getChildAtIndex", NULL);
   return retval;
 }
 
@@ -431,8 +451,7 @@ Accessible_getIndexInParent (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, -1);
 
-  retval = (long)
-    Accessibility_Accessible_getIndexInParent (CSPI_OBJREF (obj), cspi_ev ());
+  retval = Accessibility_Accessible_getIndexInParent (CSPI_OBJREF (obj), cspi_ev ());
 
   cspi_return_val_if_ev ("getIndexInparent", -1); 
   return retval;
@@ -547,6 +566,7 @@ Accessible_getStateSet (Accessible *obj)
     Accessibility_Accessible_getState (CSPI_OBJREF (obj),
 					  cspi_ev ()));
 
+  cspi_return_val_if_ev ("getState", NULL);
   return retval;
 }
 
@@ -1025,8 +1045,7 @@ AccessibleRelation_getNTargets (AccessibleRelation *obj)
   int retval;
   
   cspi_return_val_if_fail (obj, -1);
-  retval = (int)
-    Accessibility_Relation_getNTargets (CSPI_OBJREF (obj), cspi_ev());
+  retval = Accessibility_Relation_getNTargets (CSPI_OBJREF (obj), cspi_ev());
   cspi_return_val_if_ev ("getNTargets", -1);
   return retval;
 }
@@ -1052,7 +1071,7 @@ AccessibleRelation_getTarget (AccessibleRelation *obj, int i)
 
   retval = cspi_object_add (
 			 Accessibility_Relation_getTarget (CSPI_OBJREF(obj),
-							   (CORBA_short) i, cspi_ev()));
+							   i, cspi_ev()));
   cspi_return_val_if_ev ("getTarget", NULL);
   return retval;
 }
@@ -1111,7 +1130,7 @@ AccessibleStateSet_contains (AccessibleStateSet *obj,
 
   cspi_return_val_if_ev ("contains", FALSE);
 
-  return (SPIBoolean) retval;
+  return retval;
 }
 
 /**
@@ -1133,7 +1152,7 @@ AccessibleStateSet_add (AccessibleStateSet *obj,
 
   spi_state = spi_state_type_from_accessible_state (state);
   Accessibility_StateSet_add (CSPI_OBJREF (obj), spi_state, cspi_ev ());
-  cspi_check_ev ("add");
+  cspi_check_ev ("StateSet_add");
 }
 
 /**
@@ -1155,7 +1174,7 @@ AccessibleStateSet_remove (AccessibleStateSet *obj,
 
   spi_state = spi_state_type_from_accessible_state (state);
   Accessibility_StateSet_remove (CSPI_OBJREF (obj), spi_state, cspi_ev ());
-  cspi_check_ev ("remove");
+  cspi_check_ev ("StateSet_remove");
 }
 
 /**
@@ -1207,12 +1226,15 @@ AccessibleStateSet *
 AccessibleStateSet_compare (AccessibleStateSet *obj,
                             AccessibleStateSet *obj2)
 {
-  Accessibility_StateSet retval;
+  AccessibleStateSet *retval;
 
   cspi_return_val_if_fail (obj != NULL, NULL);
   cspi_return_val_if_fail (obj2 != NULL, NULL);
-  retval = Accessibility_StateSet_compare (CSPI_OBJREF(obj), CSPI_OBJREF(obj2), cspi_ev ());
-  return NULL;	
+  retval = cspi_object_add (
+			 Accessibility_StateSet_compare (CSPI_OBJREF(obj), CSPI_OBJREF(obj2), cspi_ev ()));
+
+  cspi_return_val_if_ev ("compare", NULL);
+  return retval;
 }
 
 /**
@@ -1232,7 +1254,9 @@ AccessibleStateSet_isEmpty (AccessibleStateSet *obj)
 
   cspi_return_val_if_fail (obj != NULL, FALSE);
   retval = Accessibility_StateSet_isEmpty (CSPI_OBJREF (obj), cspi_ev ());
-  return (SPIBoolean) retval;
+
+  cspi_return_val_if_ev ("isEmpty", TRUE);
+  return retval;
 }
 
 
