@@ -1,3 +1,25 @@
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
+ * Copyright 2001 Sun Microsystems Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <cspi/spi-private.h>
 
 /**
@@ -75,7 +97,7 @@ AccessibleHypertext_getLink (AccessibleHypertext *obj,
 
   retval = cspi_object_add (
     Accessibility_Hypertext_getLink (CSPI_OBJREF (obj),
-				     (CORBA_long) linkIndex,
+				     linkIndex,
 				     cspi_ev ()));
   
   cspi_return_val_if_ev ("getLink", NULL); 
@@ -105,7 +127,7 @@ AccessibleHypertext_getLinkIndex (AccessibleHypertext *obj,
 
   retval = 
     Accessibility_Hypertext_getLinkIndex (CSPI_OBJREF (obj),
-					  (CORBA_long) characterOffset,
+					  characterOffset,
 					  cspi_ev ());
 
   cspi_return_val_if_ev ("getLinkIndex", -1);
