@@ -36,16 +36,23 @@ spi_init_state_type_tables (void)
 {
   gint i;
 
+  if (accessible_state_types || atk_state_types)
+    return FALSE;
+  if (!accessible_state_types)
+    accessible_state_types = g_new (Accessibility_StateType, ATK_STATE_LAST_DEFINED);
+  if (!atk_state_types)
+    atk_state_types = g_new (AtkStateType, Accessibility_STATE_LAST_DEFINED);
   g_return_val_if_fail (accessible_state_types, FALSE);
   g_return_val_if_fail (atk_state_types, FALSE);
   
-  accessible_state_types = g_new (Accessibility_StateType, ATK_STATE_LAST_DEFINED);
-  atk_state_types = g_new (AtkStateType, ATK_STATE_LAST_DEFINED);
-
-  for (i = 0; i < ATK_STATE_LAST_DEFINED; i++)
+  for (i = 0; i < Accessibility_STATE_LAST_DEFINED; i++)
     {
       atk_state_types[i] = ATK_STATE_INVALID;
-      accessible_state_types[i] = Accessibility_STATE_INVALID;
+    }
+
+  for (i=0; i < ATK_STATE_LAST_DEFINED; i++)
+    {
+      accessible_state_types[i] = Accessibility_STATE_INVALID;	
     }
 
   accessible_state_types[ATK_STATE_ACTIVE] = Accessibility_STATE_ACTIVE;
@@ -160,8 +167,7 @@ impl_contains (PortableServer_Servant servant,
   AtkStateSet *set = atk_state_set_from_servant (servant);
 
   g_return_val_if_fail (set, FALSE);
-  return (CORBA_boolean)
-    atk_state_set_contains_state (set, atk_state_types[state]);
+  return atk_state_set_contains_state (set, atk_state_types[state]);
 }
 
 
@@ -210,8 +216,7 @@ impl_equals (PortableServer_Servant servant,
   g_object_unref (G_OBJECT(set2));
   if (return_set)
     {
-      rv = (CORBA_boolean)
-	atk_state_set_is_empty (return_set);
+      rv = atk_state_set_is_empty (return_set);
       g_object_unref (G_OBJECT(return_set));
     }
   else
@@ -250,8 +255,7 @@ impl_isEmpty (PortableServer_Servant servant,
   AtkStateSet *set = atk_state_set_from_servant (servant);
 
   g_return_val_if_fail (set, TRUE);
-  return (CORBA_boolean)
-    atk_state_set_is_empty (set);
+  return atk_state_set_is_empty (set);
 }
 
 
@@ -347,6 +351,7 @@ spi_state_set_class_init (SpiStateSetClass *klass)
 {
   POA_Accessibility_StateSet__epv *epv = &klass->epv;
 
+  spi_init_state_type_tables ();
   epv->contains = impl_contains;
   epv->add = impl_add;
   epv->remove = impl_remove;
